#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#	Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import subprocess, sys, os

# version of the OpenShot Build Wizard
version = "1.0.6"

# get path of home directory
home_dir = os.path.expanduser("~")
hasOpenShot = os.path.exists(os.path.join(home_dir, "openshot"))
ubuntu_version = "3"  # 1 = 8.04,  2 = 8.10,  3 = 9.04

# get the path of this script
path = os.path.dirname(os.path.abspath(sys.argv[0]))



if os.path.join(home_dir, "openshot_wizard") != path:
	print "--------------------------------------------"
	print "Sorry, but you have not extracted the OpenShot Build Wizard to"
	print "the correct folder.  Please be sure to run the install.py file"
	print "from the following location: %s." % os.path.join(home_dir, "openshot_wizard")
	print "--------------------------------------------"
	
else:
	
	print "\nWelcome to the OpenShot Build Wizard %s... for Ubuntu!" % version
	print "--------------------------------------------"
	print "We are going to attempt to download and build MLT and all of the "
	print "dependencies that OpenShot requires.  If you are not using"
	print "Ubuntu, you will need to modify this .py file to work for "
	print "your distribution.  Good luck!"
	print "--------------------------------------------"
	print "WARNING:  This will DELETE any existing version of ~/ffmpeg"
	print "WARNING:  This will DELETE any existing version of ~/x264"
	print "WARNING:  This will DELETE any existing version of ~/frei0r"
	print "WARNING:  This will DELETE any existing version of ~/mlt"
	print "WARNING:  This will DELETE any existing version of ~/openshot"
	print ""
	print "***  ANY EDITS TO THESE FILES & FOLDERS WILL BE OVERWRITTEN ***"
	print "--------------------------------------------"
	
	print "\nWhat version of Ubuntu are you running?\n"
	print "[1] 8.04 (32 bit) "
	print "[2] 8.10 (32 bit) "
	print "[3] 9.04 (32 bit)\n"

	print "[4] 8.04 (64 bit) "
	print "[5] 8.10 (64 bit) "
	print "[6] 9.04 (64 bit)\n"
	ubuntu_version = str.lower(raw_input("Please choose a version [1,2,3]: "))
	
	
	ready_to_begin = 'n'
	update_openshot = 'n'
	
	# prompt user for Y or N
	if hasOpenShot == False:
		ready_to_begin = str.lower(raw_input("Do you want to continue [Y/n]? "))
	else:
		print "--------------------------------------------"
		print "\nOpenShot is already installed.\n"
		print "[1] (RECOMMENDED) Get latest OpenShot source code"
		print "[2] Re-install OpenShot, MLT, Frei0r, x264, and FFmpeg"
		print "[3] Quit\n"
		update_openshot = str.lower(raw_input("Please choose an option [1,2,3]: "))
		
		# Decide what to do
		if update_openshot in ('1'):
			# Update OpenShot Source Code
			# delete folder
			subprocess.call('sudo rm ~/openshot -r', shell=True)
			
			# download newest version of openshot
			subprocess.call('bzr branch lp:openshot ~/openshot', shell=True)
		
		elif update_openshot in ('2'):
			# re-install everything
			ready_to_begin = 'y'
		else:
			# quit
			print "Good bye"
			ready_to_begin = 'n'
		
	
	# are they ready?
	if ready_to_begin in ('y', 'yes'):
	
		print "--------------------------------------------"
		print "Installing git-core, subversion, and bzr packages"
		subprocess.call('sudo apt-get install git-core subversion bzr', shell=True)
		
		print "--------------------------------------------"
		print "Downloading MLT project from GIT"
		# check if mlt folder exists
		if os.path.exists(os.path.join(home_dir, "mlt")):
			# prompt to delete folder
			delete_dir = str.lower(raw_input("Do you want to Replace the ~/mlt Directory [Y/n]? "))
			if delete_dir in ('y', 'yes'):
				# delete mlt directory
				subprocess.call('sudo rm ~/mlt -r', shell=True)
			
				# download newest version of mlt
				subprocess.call('git clone git://mltframework.org/mlt.git ~/mlt', shell=True)
		else:
			# download newest version of mlt
			subprocess.call('git clone git://mltframework.org/mlt.git ~/mlt', shell=True)
		
		print "--------------------------------------------"
		print "Downloading frei0r project from GIT"
		# check if frei0r folder exists
		if os.path.exists(os.path.join(home_dir, "frei0r")):
			# prompt to delete folder
			delete_frei0r = str.lower(raw_input("Do you want to Replace the ~/frei0r Directory [Y/n]? "))
			if delete_frei0r in ('y', 'yes'):
				# delete folder
				subprocess.call('sudo rm ~/frei0r -r', shell=True)
				
				# download newest version of frei0r
				subprocess.call('git clone git://code.dyne.org/frei0r.git ~/frei0r', shell=True)
		else:
			# download newest version of frei0r
			delete_frei0r = 'y'
			subprocess.call('git clone git://code.dyne.org/frei0r.git ~/frei0r', shell=True)
			
			
			
		print "--------------------------------------------"
		print "Downloading x264 project from GIT"
		# check if frei0r folder exists
		if os.path.exists(os.path.join(home_dir, "x264")):
			# prompt to delete folder
			delete_x264 = str.lower(raw_input("Do you want to Replace the ~/x264 Directory [Y/n]? "))
			if delete_x264 in ('y', 'yes'):
				# delete folder
				subprocess.call('sudo rm ~/x264 -r', shell=True)
				
				# download newest version of frei0r
				subprocess.call('git clone git://git.videolan.org/x264.git ~/x264', shell=True)
		else:
			# download newest version of frei0r
			delete_x264 = 'y'
			subprocess.call('git clone git://git.videolan.org/x264.git ~/x264', shell=True)
			
			
		
		print "--------------------------------------------"
		print "Downloading OpenShot from bazaar"
		# check if openshot folder exists
		if os.path.exists(os.path.join(home_dir, "openshot")):
			# prompt to delete folder
			delete_dir = str.lower(raw_input("Do you want to Replace the ~/openshot Directory [Y/n]? "))
			if delete_dir in ('y', 'yes'):
				# delete folder
				subprocess.call('sudo rm ~/openshot -r', shell=True)
				
				# download newest version of openshot
				subprocess.call('bzr branch lp:openshot ~/openshot', shell=True)
		else:
			# download newest version of openshot
			subprocess.call('bzr branch lp:openshot ~/openshot', shell=True)
			
			
		# update repository
		subprocess.call('sudo apt-get update --fix-missing', shell=True)
		
		# install the following packages from apt-get
		package_list = ["build-essential python-dev libgtk2.0-dev swig autoconf libtool automake libbz2-dev yasm checkinstall", 
						"python-pygoocanvas libsdl1.2-dev libdv4-dev libxml2-dev sox libsox-dev libvorbis-dev libtheora-dev libsamplerate0-dev kino dvgrab jack jack-rack libjack-dev ladspa-sdk libquicktime1 libquicktime-dev libfaac-dev"
						]
		
		# loop through each package, and install
		for package in package_list:
			# install each package
			print "--------------------------------------------"
			print "Installing %s\n" % package
			subprocess.call('sudo apt-get install %s' % package, shell=True)
	
	
		# only build frei0r if the user is replacing the ~/frei0r folder
		#if delete_frei0r in ('y', 'yes'):
		print "--------------------------------------------"
		print "Configuring frei0r..."
		subprocess.call('cd ~/frei0r\nautoconf', shell=True)
		subprocess.call('cd ~/frei0r\n./autogen.sh', shell=True)
		subprocess.call('cd ~/frei0r\n./configure', shell=True)
		
		print "--------------------------------------------"
		print "Building and Installing frei0r..."
		subprocess.call('cd ~/frei0r\nsudo make clean', shell=True)
		subprocess.call('cd ~/frei0r\nsudo make', shell=True)
		subprocess.call('cd ~/frei0r\nsudo checkinstall -D --pkgname="openshot-frei0r" --pkgversion="1.1.22" --maintainer="Jonathan.Oomph@gmail.com" --nodoc --default --install=yes --fstrans=no', shell=True)
		subprocess.call('cd ~/frei0r\nsudo ldconfig', shell=True)
			
			
		# only build x264 if the user is replacing the ~/x264 folder
		#if delete_x264 in ('y', 'yes'):
		print "--------------------------------------------"
		print "Building and Installing x264..."
		if ubuntu_version == "1" or ubuntu_version == "4":
			# Ubuntu 8.04 
			subprocess.call('cd ~/x264\n./configure --enable-shared --disable-asm', shell=True)
		else:
			# Newer Ubuntu versions
			subprocess.call('cd ~/x264\n./configure --enable-shared', shell=True)
		subprocess.call('cd ~/x264\nsudo make clean', shell=True)
		subprocess.call('cd ~/x264\nsudo make', shell=True)
		subprocess.call('cd ~/x264\nsudo checkinstall -D --pkgname="openshot-x264" --pkgversion="0.67.1173" --maintainer="Jonathan.Oomph@gmail.com" --nodoc --default --install=yes --fstrans=no', shell=True)
		subprocess.call('cd ~/x264\nsudo ldconfig', shell=True)
		
	
		print "\n--------------------------------------------"
		print "FFmpeg is a complete, cross-platform solution to record, convert "
		print "and stream audio and video.  If OpenShot fails to play video or "
		print "audio, it's most likely related to this step in the build process."
		print "If for some reason option 1 doesn't work, try option 2, and then 3."
		print ""
		print "You have 3 choices about the version of FFmpeg we use in OpenShot:"
		print ""
		print "[1] (RECOMMENDED) Use the default version of FFmpeg that comes with your OS"
		print "[2] Let MLT download and install the newest version, and statically link"
		print "[3] Download and install the newest GIT version FFmpeg, and replace the "
		print "       default version that comes with your OS (Required for AVCHD)"
		print ""
		ffmpeg_method = str.lower(raw_input("Which version of FFmpeg do you want to use [1,2,3]?"))
		additional_mlt_options = ""
		
		if ffmpeg_method == "1":
			# install the default FFmpeg from the repository
			print "--------------------------------------------"
			print "Install the default FFmpeg from the repository"
			subprocess.call('sudo apt-get install ffmpeg', shell=True)
			subprocess.call('sudo apt-get install libavcodec-dev libavformat-dev libswscale-dev libavutil-dev', shell=True)
			subprocess.call('sudo apt-get install libavdevice-dev', shell=True)
		
		elif ffmpeg_method == "2":
			# add some additional mlt configure options
			additional_mlt_options = " --avformat-svn "
		
		elif ffmpeg_method == "3":
			print "--------------------------------------------"
			print "Downloading latest ffmpeg"
			# remove previous versions from apt-get
			subprocess.call("sudo apt-get remove ffmpeg libavcodec-dev libavformat-dev libswscale-dev libavutil-dev", shell=True)
			subprocess.call("sudo apt-get remove libavdevice-dev", shell=True)
			subprocess.call("sudo apt-get remove x264 libx264-dev", shell=True)
			
			
			# check for existing ffmpeg folder
			if os.path.exists(os.path.join(home_dir, "ffmpeg")):
				# delete folder
				subprocess.call('sudo rm ~/ffmpeg -r', shell=True)
				
			# download newest ffmpeg source code
			subprocess.call('git clone git://git.ffmpeg.org/ffmpeg/ ~/ffmpeg', shell=True)
			subprocess.call('git clone git://git.ffmpeg.org/libswscale/ ~/ffmpeg/libswscale', shell=True)
			
			# build ffmpeg
			print "--------------------------------------------"
			print "Configuring & Building ffmpeg"



			# if Ubuntu 32 bit mode ?
			if ubuntu_version == "1":
				# 32 bit (8.04)
				subprocess.call('cd ~/ffmpeg\n./configure --enable-gpl --enable-nonfree --enable-shared --enable-pthreads --enable-libx264 --enable-libfaac --enable-libtheora', shell=True)
			
			elif ubuntu_version == "2" or ubuntu_version == "3":
				# 32 bit (8.10 & 9.04)
				# don't compile our copy, since it doesn't seem to work... just use the avformat-svn option for mlt
				# add some additional mlt configure options
				additional_mlt_options = " --avformat-svn "

			else:
				# 64 bit (all versions)
				subprocess.call('cd ~/ffmpeg\n./configure --enable-gpl --enable-nonfree --enable-shared --enable-pthreads --enable-libx264 --enable-libfaac --enable-libtheora --extra-cflags=-fPIC --extra-cflags=-DPIC', shell=True)


			if additional_mlt_options == "":
				subprocess.call('cd ~/ffmpeg\nsudo make clean', shell=True)
				subprocess.call('cd ~/ffmpeg\nsudo make', shell=True)
				subprocess.call('cd ~/ffmpeg\nsudo checkinstall -D --pkgname="openshot-ffmpeg" --pkgversion="git-2623d8f" --maintainer="Jonathan.Oomph@gmail.com" --nodoc --default --install=yes --fstrans=no', shell=True)
				subprocess.call('cd ~/ffmpeg\nsudo ldconfig', shell=True)
	
		# configure mlt
		print "\n--------------------------------------------"
		print "\nConfiguring MLT..."
		subprocess.call("cd ~/mlt\n./configure --enable-gpl --enable-nonfree --disable-qimage --disable-sox --avformat-swscale" + additional_mlt_options, shell=True)
	
		# prompt user for Y or N
		print "--------------------------------------------"
		mlt_ready = str.lower(raw_input("If there are NO 'Configuring errors or warnings' above,\n enter Yes to continue [Y/n]? "))
		
		
		# should we continue?
		if mlt_ready in ('y', 'yes'):
			print "--------------------------------------------"
			print "Building and Installing MLT..."
			subprocess.call('cd ~/mlt\nsudo make clean', shell=True)
			subprocess.call('cd ~/mlt\nsudo make', shell=True)
			subprocess.call('cd ~/mlt\nsudo checkinstall -D --pkgname="openshot-mlt" --pkgversion="0.4.3" --maintainer="Jonathan.Oomph@gmail.com" --nodoc --default --install=yes --fstrans=no', shell=True)
			subprocess.call('cd ~/mlt\nsudo ldconfig', shell=True)
		
			# copy the MLT compiled libraries to the /usr/lib folder
			print "--------------------------------------------"
			print "Copying compiled MLT libraries to /usr/lib..."
			subprocess.call('sudo rm /usr/lib/libmlt*', shell=True)
			subprocess.call('sudo cp ~/mlt/src/framework/libmlt* /usr/lib', shell=True)
			subprocess.call('sudo cp ~/mlt/src/mlt++/libmlt* /usr/lib', shell=True)
			
		
			print "--------------------------------------------"
			print "Using Swig to generate the Python Bindings..."
			subprocess.call('cd ~/mlt/src/swig/python\nsudo bash build', shell=True)
			
			# loop through each item in the Python path list
			for path in sys.path:
				# does this contain 'site-packages'
				if str.lower(path).find("site-packages") > 0 or str.lower(path).find("dist-packages") > 0:
	
					print "--------------------------------------------"
					print "Copying MLT python bindings to: %s" % path
					
					# copy the bindings into the site-packages folder
					subprocess.call('sudo cp ~/mlt/src/swig/python/mlt.py %s' % path, shell=True)
					subprocess.call('sudo cp ~/mlt/src/swig/python/_mlt.so %s' % path, shell=True)
					subprocess.call('sudo cp ~/mlt/src/swig/python/mlt_wrap.o %s' % path, shell=True)
					
					# stop looping
					break
	
	
			# Create a launcher file for the Desktop
			print "--------------------------------------------"
			print "Create custom launcher.txt file"
			
			try:
			
				launcher = ["[Desktop Entry]",
								"Version=1.0",
								"Type=Application",
								"Terminal=true",
								"Icon[en_US]=" + os.path.join(home_dir, "openshot_wizard", "openshot_logo.png"),
								"Name[en_US]=OpenShot Video Editor",
								"Exec=python " + os.path.join(home_dir, "openshot", "main", "OpenShot.py"),
								"Comment[en_US]=Create and Edit Videos",
								"Name=OpenShot Video Editor",
								"Categories=GNOME;Application;AudioVideo",
								"Comment=Create and Edit Videos",
								"Icon=" + os.path.join(home_dir, "openshot_wizard", "openshot_logo.png"),
				]
		
				f = open(os.path.join(home_dir, "openshot_wizard", "launcher.txt"), "w")
				f.write('\n'.join(launcher))
				f.close()
		
				print "--------------------------------------------"
				print "Add OpenShot launcher to the Main Menu and Desktop in Gnome"
				subprocess.call('sudo cp ~/openshot_wizard/launcher.txt /usr/share/applications/openshot.desktop', shell=True)
				subprocess.call('cp ~/openshot_wizard/launcher.txt ' + os.path.join(home_dir, "Desktop", "openshot.desktop"), shell=True)
	
			except:
				print "*** Error creating the Launcher.txt file ***"
				
	
			print "--------------------------------------------"
			print "--------------------------------------------"
			print "INSTRUCTIONS:  Be sure to scroll up and check for any build errors."
			print "To test if everything is installed correctly, try the following commands:"
			print ""
			print "$ ffplay PATH_TO_A_VIDEO "
			print "  (This should open up a new window, and play the video file)"
			print ""
			print "$ melt -query producers"
			print "  (This should include 'avformat' in the list, or MLT is not"
			print "  configured correctly.)"
			print ""
			print "$ cd ~/mlt/demo"
			print "$ sudo bash mlt_watermark"
			print "  (This should open up a new window, and show a large blue 'OK' icon)"
			print "  (If you follow the 'README' instructions in the demo folder, it "
			print "  will play a video with a watermark in the top right corner.)"
			print ""
			print "$ python"
			print ">> import mlt"
			print "  (Be sure to open a NEW terminal window. This will start the "
			print "  Python shell, and import the mlt library. If you get no "
			print "  errors, it worked!)"
			print ""
			print "$ python ~/openshot/main/OpenShot.py"
			print "  (This will run OpenShot.py.  Try dragging and dropping a few video and "
			print "  audio files into your project, and then onto the timeline.  Hit 'Play'"
			print "  and see if it works.)"
			print "--------------------------------------------"
			print "--------------------------------------------"

	
	